@@ -57,6 +57,7 @@ GST_ELEMENT_DETAILS ("mpeg1 and mpeg2 video decoder",
  */
 #define WARN_THRESHOLD (5)
 
+//#define enable_user_data
 #ifdef enable_user_data
 static GstStaticPadTemplate user_data_template_factory =
 GST_STATIC_PAD_TEMPLATE ("user_data",
@@ -78,7 +79,7 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-raw-yuv, "
-        "format = (fourcc) { YV12, I420, Y42B }, "
+        "format = (fourcc) { YV12, I420, Y42B, Y444 }, "
         "width = (int) [ 16, 4096 ], "
         "height = (int) [ 16, 4096 ], "
         "framerate = (fraction) [ 0/1, 2147483647/1 ]")
@@ -95,8 +96,11 @@ static void gst_mpeg2dec_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_mpeg2dec_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
+
+#ifndef GST_DISABLE_INDEX
 static void gst_mpeg2dec_set_index (GstElement * element, GstIndex * index);
 static GstIndex *gst_mpeg2dec_get_index (GstElement * element);
+#endif
 
 static gboolean gst_mpeg2dec_src_event (GstPad * pad, GstEvent * event);
 static const GstQueryType *gst_mpeg2dec_get_src_query_types (GstPad * pad);
@@ -112,6 +116,7 @@ static GstStateChangeReturn gst_mpeg2dec_change_state (GstElement * element,
     GstStateChange transition);
 
 static gboolean gst_mpeg2dec_sink_event (GstPad * pad, GstEvent * event);
+static gboolean gst_mpeg2dec_setcaps (GstPad * pad, GstCaps * caps);
 static GstFlowReturn gst_mpeg2dec_chain (GstPad * pad, GstBuffer * buf);
 
 static void clear_buffers (GstMpeg2dec * mpeg2dec);
@@ -193,8 +198,10 @@ gst_mpeg2dec_class_init (GstMpeg2decClass * klass)
   gobject_class->finalize = gst_mpeg2dec_finalize;
 
   gstelement_class->change_state = gst_mpeg2dec_change_state;
+#ifndef GST_DISABLE_INDEX
   gstelement_class->set_index = gst_mpeg2dec_set_index;
   gstelement_class->get_index = gst_mpeg2dec_get_index;
+#endif
 }
 
 static void
@@ -211,6 +218,8 @@ gst_mpeg2dec_init (GstMpeg2dec * mpeg2dec)
 #endif
   gst_pad_set_event_function (mpeg2dec->sinkpad,
       GST_DEBUG_FUNCPTR (gst_mpeg2dec_sink_event));
+  gst_pad_set_setcaps_function (mpeg2dec->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_mpeg2dec_setcaps));
   gst_element_add_pad (GST_ELEMENT (mpeg2dec), mpeg2dec->sinkpad);
 
   mpeg2dec->srcpad =
@@ -281,6 +290,7 @@ gst_mpeg2dec_qos_reset (GstMpeg2dec * mpeg2dec)
   GST_OBJECT_UNLOCK (mpeg2dec);
 }
 
+#ifndef GST_DISABLE_INDEX
 static void
 gst_mpeg2dec_set_index (GstElement * element, GstIndex * index)
 {
@@ -298,6 +308,7 @@ gst_mpeg2dec_get_index (GstElement * element)
 
   return mpeg2dec->index;
 }
+#endif
 
 /* see gst-plugins/gst/games/gstvideoimage.c, paint_setup_I420() */
 #define I420_Y_ROWSTRIDE(width) (GST_ROUND_UP_4(width))
@@ -394,7 +405,7 @@ crop_copy_i422_buffer (GstMpeg2dec * mpeg2dec, GstBuffer * input)
 static gboolean
 crop_buffer (GstMpeg2dec * mpeg2dec, GstBuffer ** buf)
 {
-  gboolean result = FALSE;
+  gboolean result = TRUE;
   GstBuffer *input = *buf;
   GstBuffer *outbuf = input;
 
@@ -440,8 +451,9 @@ gst_mpeg2dec_alloc_sized_buf (GstMpeg2dec * mpeg2dec, guint size,
 
     ret = gst_pad_alloc_buffer_and_set_caps (mpeg2dec->srcpad,
         GST_BUFFER_OFFSET_NONE, size, GST_PAD_CAPS (mpeg2dec->srcpad), obuf);
-    if (ret != GST_FLOW_OK)
+    if (ret != GST_FLOW_OK) {
       return ret;
+    }
 
     /* libmpeg2 needs 16 byte aligned buffers... test for this here
      * and if it fails only a single time create our own buffers from
@@ -537,8 +549,10 @@ gst_mpeg2dec_negotiate_format (GstMpeg2dec * mpeg2dec)
     mpeg2dec->v_offs =
         I420_V_OFFSET (mpeg2dec->decoded_width, mpeg2dec->decoded_height);
 
-  } else if (sequence->width == sequence->chroma_width ||
-      sequence->height == sequence->chroma_height) {
+  } else if ((sequence->width == sequence->chroma_width &&
+          sequence->height != sequence->chroma_height) ||
+      (sequence->width != sequence->chroma_width &&
+          sequence->height == sequence->chroma_height)) {
     gint halfsize;
 
     fourcc = GST_STR_FOURCC ("Y42B");
@@ -548,8 +562,15 @@ gst_mpeg2dec_negotiate_format (GstMpeg2dec * mpeg2dec)
     mpeg2dec->u_offs = halfsize;
     mpeg2dec->v_offs = halfsize + (halfsize / 2);
   } else {
-    g_warning ("mpeg2dec: 4:4:4 format not yet supported");
-    return (FALSE);
+    gint size;
+
+    size = mpeg2dec->decoded_width * mpeg2dec->decoded_height;
+
+    fourcc = GST_STR_FOURCC ("Y444");
+    mpeg2dec->format = MPEG2DEC_FORMAT_Y444;
+    mpeg2dec->size = size * 3;
+    mpeg2dec->u_offs = size;
+    mpeg2dec->v_offs = size * 2;
   }
 
   if (mpeg2dec->pixel_width == 0 || mpeg2dec->pixel_height == 0) {
@@ -620,12 +641,16 @@ handle_sequence (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
 
   mpeg2dec->width = info->sequence->picture_width;
   mpeg2dec->height = info->sequence->picture_height;
-  mpeg2dec->pixel_width = info->sequence->pixel_width;
-  mpeg2dec->pixel_height = info->sequence->pixel_height;
   mpeg2dec->decoded_width = info->sequence->width;
   mpeg2dec->decoded_height = info->sequence->height;
   mpeg2dec->total_frames = 0;
 
+  /* don't take the sequence PAR if we already have one from the sink caps */
+  if (!mpeg2dec->have_par) {
+    mpeg2dec->pixel_width = info->sequence->pixel_width;
+    mpeg2dec->pixel_height = info->sequence->pixel_height;
+  }
+
   /* mpeg2 video can only be from 16x16 to 4096x4096. Everything
    * else is a corrupted files */
   if (mpeg2dec->width > 4096 || mpeg2dec->width < 16 ||
@@ -917,12 +942,14 @@ handle_slice (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
       picture->nb_fields, GST_BUFFER_OFFSET (outbuf),
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)));
 
+#ifndef GST_DISABLE_INDEX
   if (mpeg2dec->index) {
     gst_index_add_association (mpeg2dec->index, mpeg2dec->index_id,
         (key_frame ? GST_ASSOCIATION_FLAG_KEY_UNIT : 0),
         GST_FORMAT_BYTES, GST_BUFFER_OFFSET (outbuf),
         GST_FORMAT_TIME, GST_BUFFER_TIMESTAMP (outbuf), 0);
   }
+#endif
 
   if (picture->flags & PIC_FLAG_SKIP)
     goto skip;
@@ -1106,6 +1133,11 @@ gst_mpeg2dec_chain (GstPad * pad, GstBuffer * buf)
     GST_DEBUG_OBJECT (mpeg2dec, "parse state %d", state);
 
     switch (state) {
+#if MPEG2_RELEASE >= MPEG2_VERSION (0, 5, 0)
+      case STATE_SEQUENCE_MODIFIED:
+        GST_DEBUG_OBJECT (mpeg2dec, "sequence modified");
+        /* fall through */
+#endif
       case STATE_SEQUENCE:
         ret = handle_sequence (mpeg2dec, info);
         /* if there is an error handling the sequence
@@ -1261,9 +1293,11 @@ gst_mpeg2dec_sink_event (GstPad * pad, GstEvent * event)
       break;
     }
     case GST_EVENT_EOS:
+#ifndef GST_DISABLE_INDEX
       if (mpeg2dec->index && mpeg2dec->closed) {
         gst_index_commit (mpeg2dec->index, mpeg2dec->index_id);
       }
+#endif
       ret = gst_pad_push_event (mpeg2dec->srcpad, event);
       break;
     default:
@@ -1285,6 +1319,25 @@ newseg_wrong_format:
   }
 }
 
+static gboolean
+gst_mpeg2dec_setcaps (GstPad * pad, GstCaps * caps)
+{
+  GstMpeg2dec *mpeg2dec;
+  GstStructure *s;
+
+  mpeg2dec = GST_MPEG2DEC (gst_pad_get_parent (pad));
+
+  s = gst_caps_get_structure (caps, 0);
+
+  /* parse the par, this overrides the encoded par */
+  mpeg2dec->have_par = gst_structure_get_fraction (s, "pixel-aspect-ratio",
+      &mpeg2dec->pixel_width, &mpeg2dec->pixel_height);
+
+  gst_object_unref (mpeg2dec);
+
+  return TRUE;
+}
+
 static gboolean
 gst_mpeg2dec_sink_convert (GstPad * pad, GstFormat src_format, gint64 src_value,
     GstFormat * dest_format, gint64 * dest_value)
@@ -1526,6 +1579,7 @@ gst_mpeg2dec_get_event_masks (GstPad * pad)
 }
 #endif
 
+#ifndef GST_DISABLE_INDEX
 static gboolean
 index_seek (GstPad * pad, GstEvent * event)
 {
@@ -1592,7 +1646,7 @@ index_seek (GstPad * pad, GstEvent * event)
   }
   return FALSE;
 }
-
+#endif
 
 static gboolean
 normal_seek (GstPad * pad, GstEvent * event)
@@ -1678,9 +1732,11 @@ gst_mpeg2dec_src_event (GstPad * pad, GstEvent * event)
     case GST_EVENT_SEEK:{
       gst_event_ref (event);
       if (!(res = gst_pad_push_event (mpeg2dec->sinkpad, event))) {
+#ifndef GST_DISABLE_INDEX
         if (mpeg2dec->index)
           res = index_seek (pad, event);
         else
+#endif
           res = normal_seek (pad, event);
       }
       gst_event_unref (event);
