@@ -964,3 +964,10 @@ gst_real_audio_demux_change_state (GstElement * element,
 
   return ret;
 }
+
+gboolean
+gst_rademux_plugin_init (GstPlugin * plugin)
+{
+  return gst_element_register (plugin, "rademux",
+      GST_RANK_SECONDARY, GST_TYPE_REAL_AUDIO_DEMUX);
+}
